/*
 * Email Notify
 * Copyright: (C) 2008 by Antonio Nastasi
 * Email: sifcenter (at) gmail.it
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "emailchecker.h"

#include <QStringList>
#include <kdebug.h>

#define INFORMATION_DEBUG

EmailChecker::EmailChecker()
{
  kDebug() << "Init";
  connect( this, SIGNAL( finished() ), this, SLOT( threadFinished() ) );
}



void EmailChecker::threadFinished()
{
    kDebug() << "Finish thread";
    closeConnection();
}



EmailChecker::~EmailChecker()
{
  kDebug() << "DESTROYED";
}



void EmailChecker::run()
{
  // Connect the socket signals
  connect( &socket, SIGNAL( encrypted() ), this, SLOT( socketEncrypted() ) );
  connect( &socket, SIGNAL( connected() ), this, SLOT( socketConnected() ) );
  connect( &socket, SIGNAL( readyRead() ), this, SLOT( getMessage() ) );
  connect( &socket, SIGNAL( error( QAbstractSocket::SocketError ) ),
              this, SLOT( errorConnection( QAbstractSocket::SocketError ) ) );

  state = NO_CONNECTED;
  kDebug() << "Thread launched";
  exec();
}



inline void EmailChecker::closeConnection()
{
  socket.close();
  state = NO_CONNECTED;
}



void EmailChecker::checkEmails()
{
  kDebug() << "check emails";
  if( state >= CONNECTED )
  {
    kDebug() << "Socket already connected";
    return;
  }

#ifdef INFORMATION_DEBUG
  kDebug() << "server:" << account_->getServer();
  kDebug() << "port:" << account_->getPort();
  kDebug() << "ssl:" << account_->isSSL();
  kDebug() << "protocol:" << account_->getProtocol();
#endif

  // Connect with encryption if in account there is ssl enabled
  if( account_->isSSL() )
  {
    socket.connectToHostEncrypted( account_->getServer() ,account_->getPort() );
  }
  else
  {
    socket.connectToHost( account_->getServer(), account_->getPort() );
  }

  // Wait for 6 seconds, and put the error if there is no connection
  if( socket.waitForConnected( 6000 ) )
  {
    kDebug() << "wait for connection: TRUE";
  }
  else
  {
    kDebug() << "wait for connection: FALSE";
    closeConnection();
    updateText( "Connection Error" );
  }
}



void EmailChecker::socketEncrypted()
{
  kDebug() << "MODE:" << socket.mode();
  state = CONNECTED;
}



void EmailChecker::socketConnected()
{
  kDebug() << "MODE:" << socket.mode();

  // If the account not require the SSL, then set the CONNECTED state
  if( ! account_->isSSL() )
  {
    state = CONNECTED;
  }
}



void EmailChecker::setAccount( Account *account )
{
  account_ = account;
}



// Send message trought the socket
void EmailChecker::sendMessage( CommandType type )
{
  QString buffer;
  kDebug() << "Buffer out:" << buffer.replace( QChar( '\n' ) , QChar() );

  switch( type )
  {
    case LOGIN:
      if( account_->getProtocol().contains( "POP3" ) )
        buffer = "USER "+ account_->getLogin() + "\n";
      else if( account_->getProtocol().contains( "IMAP" ) )
        buffer = ". login "+ account_->getLogin() + " " + account_->getPassword() + "\n";
      state = VERIFY_USER;
      break;

    case PASSWORD:
      buffer = "PASS " + account_->getPassword() + "\n";
      state = VERIFY_PASSWORD;
      break;

    case STAT:
      if( account_->getProtocol().contains( "POP3" ) )
        buffer = "STAT\n";
      else if(account_->getProtocol().contains( "IMAP" ))
        buffer = ". STATUS INBOX (UNSEEN)\r\n";
      state = CMD_STAT;
      break;

    default:
      break;
  }

  socket.write( buffer.toUtf8() );
#ifdef INFORMATION_DEBUG
  kDebug() << "Buffer out:" << buffer.replace( QChar( '\n' ) , QChar() );
#endif
}



// Read the data from socket
void EmailChecker::getMessage()
{
  // Read the entire buffer
  QByteArray buffer;
  buffer = socket.readAll();

  if( buffer.contains( "OK" ) )
  {
    switch( state )
    {
      case CONNECTED:
        kDebug() << "Protocol OK: connected!";
        sendMessage( LOGIN );
        break;

      case NO_CONNECTED:
        kDebug() << "Code error";
        break;

      case VERIFY_USER:
        kDebug() << "Verify user: user accepted";
        if(account_->getProtocol().contains( "POP3" ))
          sendMessage( PASSWORD );
        else if( account_->getProtocol().contains( "IMAP" ) )
          sendMessage( STAT );
        break;

      case VERIFY_PASSWORD:
        kDebug() << "Verify password: password accepted";
        sendMessage( STAT );
        break;

      case CMD_STAT:
        kDebug() << "Command Stat";
        bool ok;
        QString emailsText;
        int emails;

        if( account_->getProtocol().contains( "POP3" ) )
        {
          QStringList temp1 = QString(buffer).split(" ");
          emailsText = temp1.at( 1 );
          emails = temp1.at( 1 ).toInt( &ok, 10 );
        }
        else if(account_->getProtocol().contains( "IMAP" ) )
        {
          if( ! buffer.contains("UNSEEN") )
          {
            updateText( "Not supported: imap haven't UNSEEN" );
            closeConnection();
            break;
          }
          QStringList temp = QString(buffer).split( " " );
          QStringList temp2 = QString( temp[ 4 ] ).split( ")" );
          emailsText = QString( temp2[ 0 ] );
          emails = emailsText.toInt( &ok, 10 );
        }

        account_->setCurrentEmails( emails );
        closeConnection();
        break;
    }
  }
  else if ( buffer.contains( "-ERR" ) || buffer.contains( ". NO" ) || buffer.contains( ". BAD" ) )
  { //TODO more checks
    QString text;
    switch( state )
    {
      case NO_CONNECTED:
        kDebug() << "Code error";
        break;

      case CONNECTED:
      case CMD_STAT:
        kDebug() << "Protocol Error";
        text = "Protocol issues";
        break;

      case VERIFY_USER:
        kDebug() << "Verify user: user NOT accepted";
        text = "Wrong user";
        break;

      case VERIFY_PASSWORD:
        kDebug() << "Verify password: password NOT accepted";
        text = "Authentication failed";
        break;
    }
    closeConnection();
    updateText( text );
  }
#ifdef INFORMATION_DEBUG
  kDebug() << "Buffer out:" << QString( buffer ).replace( QChar( '\n' ) , QChar() );
#endif
}



void EmailChecker::updateText( QString text )
{
  kDebug() << "Text updated:" << text;
  emit updatedText( text );
}



void EmailChecker::errorConnection( QAbstractSocket::SocketError error )
{
   kDebug() << "Error" << error;
   updateText( "Connection Error" );
   closeConnection();
}
