/*
 * Email Notify
 * Copyright: (C) 2008 by Antonio Nastasi
 * Email: sifcenter (at) gmail.it
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "plasma-emailnotify.h"

#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <QString>
#include <KStandardDirs>
#include <KAction>
#include <KProcess>
#include <KMessageBox>
#include <plasma/svg.h>
#include <plasma/theme.h>
#include <plasma/containment.h>


EmailNotify::EmailNotify(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
    m_icon("emailnotify"),
    configDialog_(0)
{
  setHasConfigurationInterface(true);
  setBackgroundHints(NoBackground);
  resize( 200, 200 );
}



void EmailNotify::updater()
{
  kDebug() << "updater";
  timer_->start();
  slotCheckEmails();
  setText( text_ );
}



EmailNotify::~EmailNotify()
{
  if ( hasFailedToLaunch() )
  {
    qDebug() << "failed to launch";
  } else
  {
    emailChecker_.exit();
    delete configDialog_;
    delete media;
    delete config_;
    delete timer_;
    delete generalGroup_;
    delete account_;
  }
  kDebug() << "DESTROYED";
}



/* Open the configure dialog */
void EmailNotify::showConfigurationInterface()
{
  // if configDialog is null pointer, create it!
  if( configDialog_ == 0 )
  {
    configDialog_ = new ConfigDialog();
    connect( configDialog_, SIGNAL( configUpdated() ), account_, SLOT( slotReadConfig() ) );
    connect( configDialog_, SIGNAL( configUpdated() ), this,     SLOT( updater() ) );
  }
  // show configure dialog
  configDialog_->show();
}



void EmailNotify::playSound()
{
  kDebug() << "play sound";
  if( ! account_->getSoundFile().compare( "(Nothing)" ) || account_->getSoundFile().isEmpty() )
    return;
  QString file( KGlobal::dirs()->locate( "sound", account_->getSoundFile() ) );
  if( file.isEmpty() )
    return;
  media->setCurrentSource(file);
  media->play();
}



void EmailNotify::init()
{
  // Set the configuration
  config_ = new KConfig( "emailnotifyrc" );
  generalGroup_ = new KConfigGroup( config_, "General" );
  configDialog_ = 0;
  // Set the media object
  media = Phonon::createPlayer(Phonon::MusicCategory);
  // Create timer for check emails
  timer_ = new QTimer( this );
  connect ( timer_, SIGNAL( timeout() ), this, SLOT( slotCheckEmails() ) );

  connect( &emailChecker_, SIGNAL( updatedText( QString ) ), this, SLOT( setText( QString ) ) );

  if ( m_icon.isNull() )
  {
    // Loading error
    setFailedToLaunch( true, "Loading error!" );
  }

  // Set information for the current account
  account_ = new Account( config_ );
  connect( account_, SIGNAL( currentEmailsChanged() ), this, SLOT( slotEmailsChanged() ) );
  // Play a sound when email is received
  connect( account_, SIGNAL( newEmailReceived()     ), this, SLOT( playSound() ) );

  // Start Timer for check emails
  timer_->start( 1000 );

  // Start thread
  emailChecker_.start();
}



void EmailNotify::slotEmailsChanged()
{
  // At account settings the emails count is changed
  if( account_->getShortCaption() )
  {
    // Set short caption
    text_ = QString::number( account_->getCurrentEmails(), 10 );
  }
  else
  {
    // Set long caption
    text_ = "You have " +  QString::number( account_->getCurrentEmails(), 10 ) + " new messages";
  }

  QGraphicsItem::setToolTip("");
  setText( text_ );
}



void EmailNotify::slotCheckEmails()
{
  // Check if the config is complete, else put the alert message
  if( account_->getServer().isEmpty() || account_->getPort() == 0 ||
      account_->getProtocol().isEmpty() || account_->getLogin().isEmpty()
      || account_->getPassword().isEmpty() )
  {
    KMessageBox::error( NULL, "Configure me, please!", "No configuration" );
    setText( "Configure me, please!" );
    // Stop the timer, so this will be called when config is updated
    timer_->stop();

    kDebug() << "Timer stopped";
    return;
  }
  // Set the current account into email checker
  emailChecker_.setAccount( account_ );
  // Check emails
  kDebug() << "checkemails";
  emailChecker_.checkEmails();
  // Set the timer to personal choose
  if( account_->getInterval() != 0 )
  {
    timer_->setInterval( account_->getInterval() * 1000 );
    kDebug() << "Timer interval set to:" << timer_->interval();
  }
}



void EmailNotify::setText( QString text )
{
  kDebug() << "setText";
  if( account_->getShortCaption() )
  {
    QGraphicsItem::setToolTip( text );
  }
  else
  {
    text_ = text;
  }

  update();
}



void EmailNotify::paintInterface(QPainter *p,
        const QStyleOptionGraphicsItem */*option*/, const QRect &contentsRect )
{
  kDebug() << "update";
  // Set default alignment
  int alignment = Qt::AlignBottom | Qt::AlignHCenter;

  p->setRenderHint( QPainter::SmoothPixmapTransform) ;
  p->setRenderHint( QPainter::Antialiasing );

  // Grep and set font configuration
  if( account_->getFontColor().isValid() )
  {
    p->setPen( account_->getFontColor() );
  }
  else
  {
    p->setPen( Qt::white );
  }

  p->setFont( account_->getFont() );

  // If containment is Panel, fix the size and force short caption
  if( containment()->containmentType() == Plasma::Containment::PanelContainment )
  {
    // If long caption is currently setted force to switch to short mode
    if( ! account_->getShortCaption() )
    {
      account_->setShortCaption( true );
    }

    // Set Maximum size to force resizing
    setMaximumSize( contentsRect.height(), contentsRect.height() );
  }

  // Check the preferences for short caption: position and type
  if( account_->getShortCaption() )
  {
    switch( account_->getPosition() )
    {
      case 0:
        alignment = Qt::AlignTop | Qt::AlignRight;
        break;
      case 1:
        alignment = Qt::AlignTop | Qt::AlignLeft;
        break;
      case 2:
        alignment = Qt::AlignBottom | Qt::AlignRight;
        break;
      case 3:
        alignment = Qt::AlignBottom | Qt::AlignLeft;
        break;
    }
  }

  p->drawPixmap( 7, 0, m_icon.pixmap( ( int ) contentsRect.width(), ( int ) contentsRect.width() -14 ) );
  p->save();
  p->drawText( contentsRect,  alignment, text_ );
  p->restore();
}



// Add open client menu'
QList<QAction *> EmailNotify::contextActions()
{
  QList<QAction *> list;
  KAction *action = new KAction( KIcon( "email" ) ,"Open email client" ,this );
  list.append( action );
  connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotOpenEmailClient() ) );
  return list;
}



// Open client email
void EmailNotify::slotOpenEmailClient() const
{
  if( ! account_->getClientEmail().isEmpty() )
  {
    KProcess::execute( account_->getClientEmail() );
  }
  else
  {
    KMessageBox::information( NULL, "No email client setted", "Information" );
  }
}



void EmailNotify::constraintsUpdated( Plasma::Constraints constraints )
{
  update();
}

#include "plasma-emailnotify.moc"
